local BLOODTRADER_NPCID = 115264
local BLOOD_ITEMID = 124124
local BLOOD_ITEMTEXTURE = GetItemIcon(BLOOD_ITEMID)
local BLOOD_CACHEID = 1

local function IsBloodTrader()
  local guid = UnitGUID("npc")
  if not guid then return end
  local _, _, _, _, _, npcid = strsplit("-", guid)
  if not npcid then return end
  if tonumber(npcid) ~= BLOODTRADER_NPCID then return end
  return true
end

-- Hook GetMerchantItemInfo but only when it's called from within
-- FrameXML/MerchantFrame.lua's MerchantItemButton_OnModifiedClick() function
-- This makes the UI think that the Blood Trader only sells stacks of 1 for
-- the purposes of allowing split stacks.
local oldGetMerchantItemInfo = GetMerchantItemInfo
GetMerchantItemInfo = function(index, ...)
  -- ignore if not blood trader
  if not IsBloodTrader() then return oldGetMerchantItemInfo(index, ...) end

  -- get original values
  local name, texture, price, quantity, numAvailable, isUsable, extendedCost = oldGetMerchantItemInfo(index)
  
  -- only change quantity if GetMerchantItemInfo is being called from MerchantItemButton_OnModifiedClick
  local stack = debugstack(2,1,0)
  if stack:match("MerchantItemButton_OnModifiedClick") then quantity = 1 end
  
  -- return values with quantity possibly modified
  return name, texture, price, quantity, numAvailable, isUsable, extendedCost  
end
-- Do the same thing but with maxStack for the resource cache
local oldGetMerchantItemMaxStack = GetMerchantItemMaxStack
GetMerchantItemMaxStack = function(index, ...)
  -- ignore if not blood trader
  if not IsBloodTrader() or index ~= BLOOD_CACHEID then return oldGetMerchantItemMaxStack(index, ...) end

  -- get original values
  local maxStack = oldGetMerchantItemMaxStack(index)

  -- only change quantity if GetMerchantItemMaxStack is being called from MerchantItemButton_OnModifiedClick
  local stack = debugstack(2,1,0)
  if stack:match("MerchantItemButton_OnModifiedClick") then maxStack = MainMenuBarBackpackButton.freeSlots end
  
  -- return values with quantity possibly modified
  return maxStack
end


-- Show blood of sargeras as a currency in lower right of vendor window
hooksecurefunc("MerchantFrame_UpdateCurrencies", function ()
  -- ignore if not blood trader
  if not IsBloodTrader() then return end
  
  -- the following code mimics MerchantFrame_UpdateCurrencies in showing currencies in lower right
  -- except it assumes only 1 will be shown (instead of up to 6), and it uses Item functions instead
  -- of Currency functions
  MerchantExtraCurrencyInset:Show()
  MerchantExtraCurrencyBg:Show()
  MerchantMoneyFrame:SetPoint("BOTTOMRIGHT", -169, 8)
  MerchantMoneyFrame:Show()
  
  local tokenButton = _G["MerchantToken1"]
  -- if this button doesn't exist yet, create it and anchor it
  if ( not tokenButton ) then
    tokenButton = CreateFrame("BUTTON", "MerchantToken1", MerchantFrame, "BackpackTokenTemplate")
    tokenButton:SetPoint("BOTTOMRIGHT", -16, 8)
    tokenButton:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_RIGHT") GameTooltip:SetItemByID(BLOOD_ITEMID) end)
    tokenButton:SetScript("OnClick", function(self, button) if IsModifiedClick("CHATLINK") then HandleModifiedItemClick(select(2, GetItemInfo(BLOOD_ITEMID))) end end)
  end
  
  tokenButton.itemID = BLOOD_ITEMID
  tokenButton.icon:SetTexture(BLOOD_ITEMTEXTURE)
  tokenButton.count:SetText(GetItemCount(BLOOD_ITEMID, true) or 0)
  tokenButton:Show();
end)
local eventframe = CreateFrame("Frame")
eventframe:RegisterEvent("MERCHANT_SHOW")
eventframe:RegisterEvent("MERCHANT_CLOSED")
eventframe:SetScript("OnEvent", function(self, event, ...)
  if event == "MERCHANT_SHOW" then
    eventframe:RegisterEvent("BAG_UPDATE")
  elseif event == "MERCHANT_CLOSED" then
    eventframe:UnregisterEvent("BAG_UPDATE")
  elseif event == "BAG_UPDATE" then
    -- keep updating amount of bloods
    local tokenButton = _G["MerchantToken1"]
    if not tokenButton then return end  
    tokenButton.count:SetText(GetItemCount(BLOOD_ITEMID, true) or 0)
  end
end)


-- allow people to buy multiple caches AND removes confirmation when buying stacks
hooksecurefunc("MerchantFrame_Update", function()
  -- ignore if not blood trader
  if not IsBloodTrader() then return end
  
  -- replace splitstack functions if not already done so
  for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
    local itemButton = _G["MerchantItem"..i.."ItemButton"];
    if not itemButton.oldSplitStack then
      itemButton.oldSplitStack = itemButton.SplitStack
      
      itemButton.SplitStack = function(button, split)
        if not IsBloodTrader() then return button.oldSplitStack(button, split) end

        local _, _, _, stackCount = GetMerchantItemInfo(button:GetID())

        -- round up to multiple of stack size
        if split % stackCount ~= 0 then
          split = split + stackCount
          local overage = split % stackCount
          split = split - overage
        end
  
        -- if this is first item (resource cache), buy 1 multiple times
        if button:GetID() == BLOOD_CACHEID then
          split = min(split, MainMenuBarBackpackButton.freeSlots)
          while split > 0 do
            BuyMerchantItem(button:GetID(), 1)
            BuyMerchantItem(0, 0) -- slight delay to prevent "item is busy" errors
            BuyMerchantItem(0, 0)
            split = split - 1
          end
        -- otherwise just buy amount
        else
          BuyMerchantItem(button:GetID(), split)
        end
      end
    end
  end  
end)
